@@ -1,30 +1,46 @@
 # select prediction method, based on model-object
 #' @importFrom sjstats link_inverse
-select_prediction_method <- function(fun, model, expanded_frame, ci.lvl, type, faminfo, ppd, terms, typical, ...) {
+select_prediction_method <- function(fun,
+                                     model,
+                                     expanded_frame,
+                                     ci.lvl,
+                                     type,
+                                     faminfo,
+                                     ppd,
+                                     terms,
+                                     typical,
+                                     vcov.fun,
+                                     vcov.type,
+                                     vcov.args,
+                                     ...) {
   # get link-inverse-function
   linv <- sjstats::link_inverse(model)
+  if (is.null(linv)) linv <- function(x) x
 
   if (fun == "svyglm") {
     # survey-objects -----
     fitfram <- get_predictions_svyglm(model, expanded_frame, ci.lvl, linv, ...)
   } else if (fun == "svyglm.nb") {
     # survey-glm.nb-objects -----
-    fitfram <- get_predictions_svyglmnb(model, expanded_frame, ci.lvl, linv, ...)
+    fitfram <- get_predictions_svyglmnb(model, expanded_frame, ci.lvl, linv, fun, typical, terms, vcov.fun, vcov.type, vcov.args, ...)
   } else if (fun == "stanreg") {
     # stan-objects -----
     fitfram <- get_predictions_stan(model, expanded_frame, ci.lvl, type, faminfo, ppd, ...)
   } else if (fun == "brmsfit") {
     # brms-objects -----
     fitfram <- get_predictions_stan(model, expanded_frame, ci.lvl, type, faminfo, ppd, ...)
-  } else if (fun == "coxph") {
+  } else if (fun == "coxph" && type != "surv" && type != "cumhaz") {
     # coxph-objects -----
     fitfram <- get_predictions_coxph(model, expanded_frame, ci.lvl, ...)
+  } else if (fun == "coxph" && type %in% c("surv", "cumhaz")) {
+    # coxph-objects -----
+    fitfram <- get_predictions_survival(model, expanded_frame, ci.lvl, type, terms, ...)
   } else if (fun == "lrm") {
     # lrm-objects -----
     fitfram <- get_predictions_lrm(model, expanded_frame, ci.lvl, linv, ...)
   } else if (fun == "glmmTMB") {
     # glmmTMB-objects -----
-    fitfram <- get_predictions_glmmTMB(model, expanded_frame, ci.lvl, linv, type, ...)
+    fitfram <- get_predictions_glmmTMB(model, expanded_frame, ci.lvl, linv, type, terms, typical, ...)
   } else if (fun %in% c("lmer", "nlmer", "glmer")) {
     # merMod-objects  -----
     fitfram <- get_predictions_merMod(model, expanded_frame, ci.lvl, linv, type, terms, typical, ...)
@@ -46,6 +62,9 @@ select_prediction_method <- function(fun, model, expanded_frame, ci.lvl, type, f
   } else if (fun == "clm") {
     # clm-objects -----
     fitfram <- get_predictions_clm(model, expanded_frame, ci.lvl, linv, ...)
+  } else if (fun == "clm2") {
+    # clm2-objects -----
+    fitfram <- get_predictions_clm2(model, expanded_frame, ci.lvl, linv, ...)
   } else if (fun == "Zelig-relogit") {
     # Zelig-relogit-objects -----
     fitfram <- get_predictions_zelig(model, expanded_frame, ci.lvl, linv, ...)
@@ -54,13 +73,13 @@ select_prediction_method <- function(fun, model, expanded_frame, ci.lvl, type, f
     fitfram <- get_predictions_polr(model, expanded_frame, ci.lvl, linv, typical, terms, fun, ...)
   } else if (fun %in% c("betareg", "truncreg", "zeroinfl", "hurdle")) {
     # betareg, truncreg, zeroinfl and hurdle-objects -----
-    fitfram <- get_predictions_generic2(model, expanded_frame, ci.lvl, fun, typical, terms, ...)
+    fitfram <- get_predictions_generic2(model, expanded_frame, ci.lvl, linv, type, fun, typical, terms, vcov.fun, vcov.type, vcov.args, ...)
   } else if (fun %in% c("glm", "glm.nb", "glmRob")) {
     # glm-objects -----
-    fitfram <- get_predictions_glm(model, expanded_frame, ci.lvl, linv, ...)
+    fitfram <- get_predictions_glm(model, expanded_frame, ci.lvl, linv, typical, fun, terms, vcov.fun, vcov.type, vcov.args, ...)
   } else if (fun == "lm") {
     # lm-objects -----
-    fitfram <- get_predictions_lm(model, expanded_frame, ci.lvl, linv, ...)
+    fitfram <- get_predictions_lm(model, expanded_frame, ci.lvl, fun, typical, terms, vcov.fun, vcov.type, vcov.args, ...)
   } else {
     # general-objects -----
     fitfram <- get_predictions_generic(model, expanded_frame, linv, ...)
@@ -128,9 +147,9 @@ get_predictions_svyglm <- function(model, fitfram, ci.lvl, linv, ...) {
 
 # predictions for glm ----
 
-get_predictions_glm <- function(model, fitfram, ci.lvl, linv, ...) {
+get_predictions_glm <- function(model, fitfram, ci.lvl, linv, typical, fun, terms, vcov.fun, vcov.type, vcov.args, ...) {
   # does user want standard errors?
-  se <- !is.null(ci.lvl) && !is.na(ci.lvl)
+  se <- !is.null(ci.lvl) && !is.na(ci.lvl) && is.null(vcov.fun)
 
   # for models from "robust"-pkg (glmRob) we need to
   # suppress warnings about fake models
@@ -144,7 +163,7 @@ get_predictions_glm <- function(model, fitfram, ci.lvl, linv, ...) {
     ))
 
   # copy predictions
-  get_base_fitfram(fitfram, linv, prdat, se, ci.lvl)
+  get_base_fitfram(model, fitfram, linv, prdat, se, ci.lvl, fun, typical, terms, vcov.fun, vcov.type, vcov.args)
 }
 
 
@@ -152,7 +171,6 @@ get_predictions_glm <- function(model, fitfram, ci.lvl, linv, ...) {
 
 #' @importFrom tidyr gather
 #' @importFrom dplyr bind_cols bind_rows
-#' @importFrom tibble rownames_to_column
 #' @importFrom rlang .data
 get_predictions_polr <- function(model, fitfram, ci.lvl, linv, typical, terms, fun, ...) {
 
@@ -177,7 +195,7 @@ get_predictions_polr <- function(model, fitfram, ci.lvl, linv, typical, terms, f
   # case. just return predictions
   if (nrow(prdat) > nrow(fitfram) && ncol(prdat) == 1) {
     colnames(prdat)[1] <- "predicted"
-    return(tibble::rownames_to_column(prdat, var = "response.level"))
+    return(rownames_as_column(prdat, var = "response.level"))
   }
 
   # bind predictions to model frame
@@ -221,7 +239,7 @@ get_predictions_polr <- function(model, fitfram, ci.lvl, linv, typical, terms, f
 
 get_predictions_zelig <- function(model, fitfram, ci.lvl, linv, ...) {
 
-  message("Support for Zelig-models not implemented yet.")
+  stop("`ggpredict()` does currently not support Zelig-models.", call. = FALSE)
 
   # does user want standard errors?
   se <- !is.null(ci.lvl) && !is.na(ci.lvl)
@@ -314,12 +332,92 @@ get_predictions_clm <- function(model, fitfram, ci.lvl, linv, ...) {
 }
 
 
+# predictions for cumulative link model2 ----
+
+#' @importFrom sjmisc to_long
+#' @importFrom sjstats resp_var resp_val
+get_predictions_clm2 <- function(model, fitfram, ci.lvl, linv, ...) {
+
+  stop("`ggpredict()` does currently not support clm2-models.", call. = FALSE)
+
+  # does user want standard errors?
+  se <- !is.null(ci.lvl) && !is.na(ci.lvl)
+
+  # compute ci, two-ways
+  if (!is.null(ci.lvl) && !is.na(ci.lvl))
+    ci <- 1 - ((1 - ci.lvl) / 2)
+  else
+    ci <- .975
+
+  fitfram <- add_cols(fitfram, as.factor(sjstats::resp_val(model)), .before = 1)
+  colnames(fitfram)[1] <- sjstats::resp_var(model)
+
+  # prediction, with CI
+  prdat <-
+    stats::predict(
+      model,
+      newdata = fitfram,
+      type = "prob",
+      interval = se,
+      level = ci,
+      ...
+    )
+
+  # convert to data frame.
+  prdat <- as.data.frame(prdat)
+
+  # bind predictions to model frame
+  fitfram <- dplyr::bind_cols(prdat, fitfram)
+
+  # get levels of response
+  lv <- levels(sjstats::model_frame(model)[[sjstats::resp_var(model)]])
+
+  # for proportional ordinal logistic regression (see ordinal::clm),
+  # we have predicted values for each response category. Hence,
+  # gather columns. Since we also have conf. int. for each response
+  # category, we need to gather multiple columns at once
+
+  if (isTRUE(se)) {
+
+    # length of each variable block
+    l <- seq_len(ncol(prdat) / 3)
+    colnames(fitfram)[l] <- lv
+
+    fitfram <- sjmisc::to_long(
+      fitfram,
+      keys = "response.level",
+      values = c("predicted", "conf.low", "conf.high"),
+      l,
+      l + length(l),
+      l + 2 * length(l)
+    )
+
+  } else {
+    key_col <- "response.level"
+    value_col <- "predicted"
+
+    fitfram <- tidyr::gather(fitfram, !! key_col, !! value_col, !! 1:ncol(prdat))
+
+    # No CI
+    fitfram$conf.low <- NA
+    fitfram$conf.high <- NA
+  }
+
+  fitfram
+}
+
+
 # predictions for regression models w/o SE ----
 
-get_predictions_generic2 <- function(model, fitfram, ci.lvl, fun, typical, terms, ...) {
+#' @importFrom stats qlogis
+get_predictions_generic2 <- function(model, fitfram, ci.lvl, linv, type, fun, typical, terms, vcov.fun, vcov.type, vcov.args, ...) {
   # get prediction type.
   pt <- dplyr::case_when(
-    fun %in% c("hurdle", "zeroinfl") ~ "response",
+    fun == "zeroinfl" && type == "fe" ~ "count",
+    fun == "zeroinfl" && type == "fe.zi" ~ "response",
+    fun == "hurdle" && type == "fe" ~ "count",
+    fun == "hurdle" && type == "fe.zi" ~ "response",
+    fun == "betareg" ~ "link",
     TRUE ~ "response"
   )
 
@@ -329,6 +427,51 @@ get_predictions_generic2 <- function(model, fitfram, ci.lvl, fun, typical, terms
   else
     ci <- .975
 
+
+  # if (type == "debug") {
+  #   mf <- sjstats::model_frame(model)
+  #
+  #   newdata <- get_expanded_data(
+  #     model,
+  #     mf,
+  #     terms,
+  #     typ.fun = typical,
+  #     fac.typical = FALSE,
+  #     pretty.message = FALSE
+  #   )
+  #
+  #   condformula <- as.formula(paste0("~", deparse(stats::formula(model)[[3]][[2]])))
+  #   x.cond <- stats::model.matrix(condformula, model = "count", data = newdata)
+  #   beta.cond <- stats::coef(model, model = "count")
+  #
+  #   ziformula <- as.formula(paste0("~", deparse(stats::formula(model)[[3]][[3]])))
+  #   x.zi <- stats::model.matrix(ziformula, model = "zero", data = newdata)
+  #   beta.zi <- stats::coef(model, model = "zero")
+  #
+  #   pred.condpar.psim <- MASS::mvrnorm(1000, mu = beta.cond, Sigma = stats::vcov(model, model = "count"))
+  #   pred.cond.psim <- x.cond %*% t(pred.condpar.psim)
+  #   pred.zipar.psim <- MASS::mvrnorm(1000, mu = beta.zi, Sigma = stats::vcov(model, model = "zero"))
+  #   pred.zi.psim <- x.zi %*% t(pred.zipar.psim)
+  #   pred.ucount.psim <- exp(pred.cond.psim) * (1 - stats::plogis(pred.zi.psim))
+  #
+  #   fitfram <- newdata
+  #   fitfram$predictions <- apply(pred.ucount.psim, 1, mean)
+  #   fitfram$conf.low <- apply(pred.ucount.psim, 1, quantile, 1 - ci)
+  #   fitfram$conf.high <- apply(pred.ucount.psim, 1, quantile, ci)
+  #
+  #   grp <- rlang::syms(terms)
+  #   fitfram <- fitfram %>%
+  #     dplyr::group_by(!!! grp) %>%
+  #     dplyr::summarize(
+  #       predicted = mean(.data$predictions),
+  #       conf.low = mean(.data$conf.low),
+  #       conf.high = mean(.data$conf.high)
+  #     )
+  #
+  #   return(fitfram)
+  #
+  # }
+
   # get predictions
   prdat <-
     stats::predict(
@@ -338,16 +481,29 @@ get_predictions_generic2 <- function(model, fitfram, ci.lvl, fun, typical, terms
       ...
     )
 
+
   fitfram$predicted <- as.vector(prdat)
 
+
+  # for some models, we need some backtransformation
+
+  if (fun %in% c("zeroinfl", "hurdle")) {
+    fitfram$predicted <- log(fitfram$predicted)
+  }
+
+
   # get standard errors from variance-covariance matrix
   se.pred <-
     get_se_from_vcov(
       model = model,
       fitfram = fitfram,
       typical = typical,
+      type = type,
       terms = terms,
-      fun = fun
+      fun = fun,
+      vcov.fun = vcov.fun,
+      vcov.type = vcov.type,
+      vcov.args = vcov.args
     )
 
 
@@ -355,14 +511,16 @@ get_predictions_generic2 <- function(model, fitfram, ci.lvl, fun, typical, terms
     se.fit <- se.pred$se.fit
     fitfram <- se.pred$fitfram
     # CI
-    fitfram$conf.low <- fitfram$predicted - stats::qnorm(ci) * se.fit
-    fitfram$conf.high <- fitfram$predicted + stats::qnorm(ci) * se.fit
+    fitfram$conf.low <- linv(fitfram$predicted - stats::qnorm(ci) * se.fit)
+    fitfram$conf.high <- linv(fitfram$predicted + stats::qnorm(ci) * se.fit)
   } else {
     # CI
     fitfram$conf.low <- NA
     fitfram$conf.high <- NA
   }
 
+  fitfram$predicted <- linv(fitfram$predicted)
+
 
   fitfram
 }
@@ -410,7 +568,7 @@ get_predictions_lrm <- function(model, fitfram, ci.lvl, linv, ...) {
 
 # predictions for svyglm.nb ----
 
-get_predictions_svyglmnb <- function(model, fitfram, ci.lvl, linv, ...) {
+get_predictions_svyglmnb <- function(model, fitfram, ci.lvl, linv, fun, typical, terms, vcov.fun, vcov.type, vcov.args, ...) {
   # does user want standard errors?
   se <- !is.null(ci.lvl) && !is.na(ci.lvl)
 
@@ -424,14 +582,19 @@ get_predictions_svyglmnb <- function(model, fitfram, ci.lvl, linv, ...) {
     )
 
   # copy predictions
-  get_base_fitfram(fitfram, linv, prdat, se, ci.lvl)
+  get_base_fitfram(model, fitfram, linv, prdat, se, ci.lvl, fun, typical, terms, vcov.fun, vcov.type, vcov.args)
 }
 
 
 # predictions for glmmTMB ----
 
-#' @importFrom stats family
-get_predictions_glmmTMB <- function(model, fitfram, ci.lvl, linv, type, ...) {
+#' @importFrom stats predict qnorm family model.matrix formula terms vcov plogis simulate
+#' @importFrom sjstats model_frame
+#' @importFrom lme4 fixef nobars
+#' @importFrom MASS mvrnorm
+#' @importFrom dplyr group_by summarize
+#' @importFrom rlang syms
+get_predictions_glmmTMB <- function(model, fitfram, ci.lvl, linv, type, terms, typical, ...) {
   # does user want standard errors?
   se <- !is.null(ci.lvl) && !is.na(ci.lvl)
 
@@ -444,51 +607,169 @@ get_predictions_glmmTMB <- function(model, fitfram, ci.lvl, linv, type, ...) {
 
   # check whether predictions should be conditioned
   # on random effects (grouping level) or not.
-  if (type == "fe")
+
+  if (type %in% c("fe", "fe.zi"))
     ref <- NA
   else
     ref <- NULL
 
 
-  prdat <- stats::predict(
-    model,
-    newdata = fitfram,
-    type = "link",
-    se.fit = se,
-    # not implemented in glmmTMB <= 0.2.1
-    # re.form = ref,
-    ...
-  )
+  add.args <- lapply(match.call(expand.dots = F)$`...`, function(x) x)
+
+  if ("nsim" %in% names(add.args))
+    nsim <- eval(add.args[["nsim"]])
+  else
+    nsim <- 1000
 
 
-  # did user request standard errors? if yes, compute CI
-  if (se) {
-    fitfram$predicted <- linv(prdat$fit)
+  if (type == "fe.zi") {
 
-    # add random effect uncertainty to s.e.
-    if (type == "re" && requireNamespace("glmmTMB", quietly = TRUE)) {
-      sig <- sum(attr(glmmTMB::VarCorr(model)[[1]], "sc"))
-      lf <- get_link_fun(model)
-      prdat$se.fit <- prdat$se.fit + lf(sig^2)
+    prdat <- as.vector(stats::predict(
+      model,
+      newdata = fitfram,
+      type = "response",
+      se.fit = FALSE,
+      ## FIXME not implemented in glmmTMB <= 0.2.2
+      # re.form = ref,
+      ...
+    ))
+
+    if (!se) {
+
+      fitfram$predicted <- prdat
+      fitfram$conf.low <- NA
+      fitfram$conf.high <- NA
+
+    } else {
+
+      mf <- sjstats::model_frame(model)
+
+      newdata <- get_expanded_data(
+        model,
+        mf,
+        terms,
+        typ.fun = typical,
+        fac.typical = FALSE,
+        pretty.message = FALSE
+      )
+
+      prdat.sim <- get_glmmTMB_predictions(model, newdata, nsim)
+
+      if (is.null(prdat.sim))
+        stop("Predicted values could not be computed. Try reducing number of simulation, using argument `nsim` (e.g. `nsim = 100`)", call. = FALSE)
+
+      sims <- exp(prdat.sim$cond) * (1 - stats::plogis(prdat.sim$zi))
+      fitfram <- newdata
+
+      fitfram$predicted <- apply(sims, 1, mean)
+      fitfram$conf.low <- apply(sims, 1, quantile, probs = 1 - ci)
+      fitfram$conf.high <- apply(sims, 1, quantile, probs = ci)
+
+      grp <- rlang::syms(terms)
+      fitfram <- fitfram %>%
+        dplyr::group_by(!!! grp) %>%
+        dplyr::summarize(
+          predicted = mean(.data$predicted),
+          conf.low = mean(.data$conf.low),
+          conf.high = mean(.data$conf.high)
+        )
+
+      # we use the predicted values from "predict(type = "reponse")", but the
+      # bootstrapped CI - so we need to fix a bit here
+
+      if (length(prdat) == nrow(fitfram)) {
+        ci.range <- (fitfram$conf.high - fitfram$conf.low) / 2
+        fitfram$predicted <- prdat
+        fitfram$conf.low <- fitfram$predicted - ci.range
+        fitfram$conf.high <- fitfram$predicted + ci.range
+      }
     }
+  } else if (type == "re.zi") {
 
+    sims <- stats::simulate(model, nsim = nsim)
+    fitfram <- sjstats::model_frame(model)
+
+    fitfram$predicted <- apply(sims, 1, mean)
+    fitfram$conf.low <- apply(sims, 1, quantile, probs = 1 - ci)
+    fitfram$conf.high <- apply(sims, 1, quantile, probs = ci)
+
+    grp <- rlang::syms(terms)
+    fitfram <- fitfram %>%
+      dplyr::group_by(!!! grp) %>%
+      dplyr::summarize(
+        predicted = mean(.data$predicted),
+        conf.low = mean(.data$conf.low),
+        conf.high = mean(.data$conf.high)
+      )
 
-    # calculate CI
-    fitfram$conf.low <- linv(prdat$fit - stats::qnorm(ci) * prdat$se.fit)
-    fitfram$conf.high <- linv(prdat$fit + stats::qnorm(ci) * prdat$se.fit)
   } else {
-    # copy predictions
-    fitfram$predicted <- linv(as.vector(prdat))
+    prdat <- stats::predict(
+      model,
+      newdata = fitfram,
+      type = "link",
+      se.fit = se,
+      ## FIXME not implemented in glmmTMB <= 0.2.2
+      ## TODO once this is fixed, update docs in ggpredict, argument type
+      # re.form = ref,
+      ...
+    )
 
-    # no CI
-    fitfram$conf.low <- NA
-    fitfram$conf.high <- NA
+    # we need backtransformation for re-variance...
+    lf <- get_link_fun(model)
+
+    # did user request standard errors? if yes, compute CI
+    if (se) {
+      fitfram$predicted <- linv(prdat$fit)
+
+      # add random effect uncertainty to s.e.
+      if (type %in% c("re", "re.zi") && requireNamespace("glmmTMB", quietly = TRUE)) {
+        sig <- sum(attr(glmmTMB::VarCorr(model)[[1]], "sc"))
+        res.var <- lf(sig^2)
+        if (is.null(res.var) || is.infinite(res.var) || is.na(res.var)) res.var <- 1
+        prdat$se.fit <- prdat$se.fit + res.var + getVarRand(model)
+      }
+
+      # calculate CI
+      fitfram$conf.low <- linv(prdat$fit - stats::qnorm(ci) * prdat$se.fit)
+      fitfram$conf.high <- linv(prdat$fit + stats::qnorm(ci) * prdat$se.fit)
+    } else {
+      # copy predictions
+      fitfram$predicted <- linv(as.vector(prdat))
+
+      # no CI
+      fitfram$conf.low <- NA
+      fitfram$conf.high <- NA
+    }
   }
 
   fitfram
 }
 
 
+get_glmmTMB_predictions <- function(model, newdata, nsim) {
+  tryCatch(
+    {
+      x.cond <- stats::model.matrix(lme4::nobars(stats::formula(model)[-2]), newdata)
+      beta.cond <- lme4::fixef(model)$cond
+
+      ziformula <- model$modelInfo$allForm$ziformula
+      x.zi <- stats::model.matrix(stats::terms(ziformula), newdata)
+      beta.zi <- lme4::fixef(model)$zi
+
+      pred.condpar.psim <- MASS::mvrnorm(n = nsim, mu = beta.cond, Sigma = stats::vcov(model)$cond)
+      pred.cond.psim <- x.cond %*% t(pred.condpar.psim)
+      pred.zipar.psim <- MASS::mvrnorm(n = nsim, mu = beta.zi, Sigma = stats::vcov(model)$zi)
+      pred.zi.psim <- x.zi %*% t(pred.zipar.psim)
+
+      list(cond = pred.cond.psim, zi = pred.zi.psim)
+    },
+    error = function(x) { NULL },
+    warning = function(x) { NULL },
+    finally = function(x) { NULL }
+  )
+}
+
+
 # predictions for merMod ----
 
 get_predictions_merMod <- function(model, fitfram, ci.lvl, linv, type, terms, typical, ...) {
@@ -563,13 +844,11 @@ get_predictions_merMod <- function(model, fitfram, ci.lvl, linv, type, terms, ty
 # predictions for stan ----
 
 #' @importFrom tidyr gather
-#' @importFrom tibble as_tibble
 #' @importFrom sjstats hdi resp_var resp_val
 #' @importFrom sjmisc rotate_df
 #' @importFrom purrr map_dbl map_df
 #' @importFrom dplyr bind_cols select bind_rows n_distinct
 #' @importFrom stats median formula
-#' @importFrom tidyselect ends_with
 get_predictions_stan <- function(model, fitfram, ci.lvl, type, faminfo, ppd, ...) {
   # check if pkg is available
   if (!requireNamespace("rstantools", quietly = TRUE)) {
@@ -596,21 +875,12 @@ get_predictions_stan <- function(model, fitfram, ci.lvl, type, faminfo, ppd, ...
       fitfram[[resp.name]] <- factor(1)
     }
 
-    if (inherits(model, "brmsfit")) {
-      prdat2 <- prdat <- rstantools::posterior_predict(
-        model,
-        newdata = fitfram,
-        re_formula = ref,
-        ...
-      )
-    } else {
-      prdat2 <- prdat <- rstantools::posterior_predict(
-        model,
-        newdata = fitfram,
-        re.form = ref,
-        ...
-      )
-    }
+    prdat2 <- prdat <- rstantools::posterior_predict(
+      model,
+      newdata = fitfram,
+      re.form = ref,
+      ...
+    )
 
   } else {
     # get posterior distribution of the linear predictor
@@ -631,7 +901,7 @@ get_predictions_stan <- function(model, fitfram, ci.lvl, type, faminfo, ppd, ...
   }
 
   # we have a list of 4000 samples, so we need to coerce to data frame
-  prdat <- tibble::as_tibble(prdat)
+  prdat <- as.data.frame(prdat)
 
 
   # handle cumulative link models
@@ -662,11 +932,11 @@ get_predictions_stan <- function(model, fitfram, ci.lvl, type, faminfo, ppd, ...
     fitfram$response.level <- ""
 
     for (i in resp.vars) {
-      pos <- tidyselect::ends_with(i, vars = tmp$grp)
+      pos <- string_ends_with(pattern = i, x = tmp$grp)
 
       if (sjmisc::is_empty(pos)) {
         i <- gsub(pattern = "[\\_\\.]", replacement = "", x = i)
-        pos <- tidyselect::ends_with(i, vars = tmp$grp)
+        pos <- string_ends_with(pattern = i, x = tmp$grp)
       }
 
       fitfram$response.level[pos] <- i
@@ -753,6 +1023,71 @@ get_predictions_coxph <- function(model, fitfram, ci.lvl, ...) {
 }
 
 
+# predictions for coxph, survival and cumulative hazard ----
+
+get_predictions_survival <- function(model, fitfram, ci.lvl, type, terms, ...) {
+  # does user want standard errors?
+  se <- !is.null(ci.lvl) && !is.na(ci.lvl)
+
+  # compute ci, two-ways
+  if (!is.null(ci.lvl) && !is.na(ci.lvl))
+    ci <- 1 - ((1 - ci.lvl) / 2)
+  else
+    ci <- .975
+
+  if (!requireNamespace("survival", quietly = TRUE)) {
+    stop("Package `survival` required. Please install it.", call. = FALSE)
+  }
+
+  # get survial probabilities and cumulative hazards
+
+  prdat <- survival::survfit(
+    model,
+    newdata = fitfram,
+    se.fit = TRUE,
+    conf.int = ci,
+    ...
+  )
+
+  # check what user requested and either returnd surv probs
+  # or cumulative hazards, including CI
+
+  if (type == "surv") {
+    pr <- prdat$surv
+    lower <- prdat$lower
+    upper <- prdat$upper
+  } else {
+    pr <- prdat$cumhaz
+    lower <- pr - stats::qnorm(ci) * prdat$std.err
+    upper <- pr + stats::qnorm(ci) * prdat$std.err
+    # ugly fix...
+    pr[which(pr < 0)] <- 0
+    lower[which(lower < 0)] <- 0
+    upper[which(upper < 0)] <- 0
+  }
+
+  # Now we need the groups, as survfit() only returns numeric indices
+
+  clean_terms <- get_clear_vars(terms)
+  ff <- fitfram[clean_terms]
+
+  purrr::map_df(sjmisc::seq_row(ff), function(i) {
+
+    dat <- data.frame(
+      time = prdat$time,
+      predicted = pr[, i],
+      conf.low = lower[, i],
+      conf.high = upper[, i]
+    )
+
+    dat2 <- purrr::map(sjmisc::seq_col(ff), ~ ff[i, .x])
+    names(dat2) <- clean_terms
+    dat2 <- data.frame(dat2, stringsAsFactors = FALSE)
+
+    cbind(dat[, 1, drop = FALSE], dat2, dat[, 2:4])
+  })
+}
+
 
 # predictions for gam ----
 
@@ -817,9 +1152,9 @@ get_predictions_vgam <- function(model, fitfram, ci.lvl, linv, ...) {
 
 #' @importFrom dplyr bind_cols
 #' @importFrom tidyr gather
-get_predictions_lm <- function(model, fitfram, ci.lvl, linv, ...) {
+get_predictions_lm <- function(model, fitfram, ci.lvl, fun, typical, terms, vcov.fun, vcov.type, vcov.args, ...) {
   # does user want standard errors?
-  se <- !is.null(ci.lvl) && !is.na(ci.lvl)
+  se <- !is.null(ci.lvl) && !is.na(ci.lvl) && is.null(vcov.fun)
 
   # compute ci, two-ways
   if (!is.null(ci.lvl) && !is.na(ci.lvl))
@@ -837,7 +1172,34 @@ get_predictions_lm <- function(model, fitfram, ci.lvl, linv, ...) {
     )
 
   # did user request standard errors? if yes, compute CI
-  if (se) {
+  if (!is.null(vcov.fun)) {
+    # copy predictions
+    fitfram$predicted <- as.vector(prdat)
+
+    se.pred <-
+      get_se_from_vcov(
+        model = model,
+        fitfram = fitfram,
+        typical = typical,
+        terms = terms,
+        fun = fun,
+        vcov.fun = vcov.fun,
+        vcov.type = vcov.type,
+        vcov.args = vcov.args
+      )
+
+    if (!is.null(se.pred)) {
+      se.fit <- se.pred$se.fit
+      fitfram <- se.pred$fitfram
+      # CI
+      fitfram$conf.low <- fitfram$predicted - stats::qnorm(ci) * se.fit
+      fitfram$conf.high <- fitfram$predicted + stats::qnorm(ci) * se.fit
+    } else {
+      # CI
+      fitfram$conf.low <- NA
+      fitfram$conf.high <- NA
+    }
+  } else if (se) {
     # copy predictions
     fitfram$predicted <- prdat$fit
 
@@ -876,7 +1238,6 @@ get_predictions_lm <- function(model, fitfram, ci.lvl, linv, ...) {
 #' @importFrom stats model.matrix formula vcov
 #' @importFrom sjstats resp_var pred_vars
 #' @importFrom purrr map
-#' @importFrom tibble add_column
 get_predictions_lme <- function(model, fitfram, ci.lvl, type, terms, typical, ...) {
   # does user want standard errors?
   se <- !is.null(ci.lvl) && !is.na(ci.lvl)
@@ -1017,7 +1378,6 @@ get_predictions_multinom <- function(model, fitfram, ci.lvl, linv, typical, term
 # predictions for generic models ----
 
 #' @importFrom prediction prediction
-#' @importFrom tibble as_tibble
 #' @importFrom sjmisc var_rename
 get_predictions_generic <- function(model, fitfram, linv, ...) {
   prdat <-
@@ -1039,24 +1399,66 @@ get_predictions_generic <- function(model, fitfram, linv, ...) {
 }
 
 
-get_base_fitfram <- function(fitfram, linv, prdat, se, ci.lvl) {
+get_base_fitfram <- function(model, fitfram, linv, prdat, se, ci.lvl, fun, typical, terms, vcov.fun, vcov.type, vcov.args) {
+
   # compute ci, two-ways
+
   if (!is.null(ci.lvl) && !is.na(ci.lvl))
     ci <- 1 - ((1 - ci.lvl) / 2)
   else
     ci <- .975
 
+
   # copy predictions
+
   if (typeof(prdat) == "double")
-    fitfram$predicted <- linv(prdat)
+    .predicted <- prdat
   else
-    fitfram$predicted <- linv(prdat$fit)
+    .predicted <- prdat$fit
+
+
+  # get standard errors, if computed
+
+  if (obj_has_name(prdat, "se.fit"))
+    se.fit <- prdat$se.fit
+  else
+    se.fit <- NULL
+
+
+  fitfram$predicted <- linv(.predicted)
+
+
+  # did user request robust standard errors?
+
+  if (!is.null(vcov.fun)) {
+    se.pred <-
+      get_se_from_vcov(
+        model = model,
+        fitfram = fitfram,
+        typical = typical,
+        terms = terms,
+        fun = fun,
+        vcov.fun = vcov.fun,
+        vcov.type = vcov.type,
+        vcov.args = vcov.args
+      )
+
+    if (!is.null(se.pred)) {
+      se.fit <- se.pred$se.fit
+      fitfram <- se.pred$fitfram
+      se <- TRUE
+    } else {
+      se.fit <- NULL
+      se <- FALSE
+    }
+  }
+
 
   # did user request standard errors? if yes, compute CI
-  if (se) {
-    # calculate CI
-    fitfram$conf.low <- linv(prdat$fit - stats::qnorm(ci) * prdat$se.fit)
-    fitfram$conf.high <- linv(prdat$fit + stats::qnorm(ci) * prdat$se.fit)
+
+  if (se && !is.null(se.fit)) {
+    fitfram$conf.low <- linv(.predicted - stats::qnorm(ci) * se.fit)
+    fitfram$conf.high <- linv(.predicted + stats::qnorm(ci) * se.fit)
   } else {
     # No CI
     fitfram$conf.low <- NA
@@ -1070,14 +1472,54 @@ get_base_fitfram <- function(fitfram, linv, prdat, se, ci.lvl) {
 
 # get standard errors of predictions from model matrix and vcov ----
 
-#' @importFrom tibble add_column
+get_se_from_vcov <- function(model,
+                             fitfram,
+                             typical,
+                             terms,
+                             fun = NULL,
+                             type = "fe",
+                             vcov.fun = NULL,
+                             vcov.type = NULL,
+                             vcov.args = NULL) {
+
+  tryCatch(
+    {
+      safe_se_from_vcov(
+        model,
+        fitfram,
+        typical,
+        terms,
+        fun,
+        type,
+        vcov.fun,
+        vcov.type,
+        vcov.args
+      )
+    },
+    error = function(x) { NULL },
+    warning = function(x) { NULL },
+    finally = function(x) { NULL }
+  )
+
+}
+
 #' @importFrom stats model.matrix terms vcov formula
 #' @importFrom dplyr arrange n_distinct
-#' @importFrom sjstats resp_var model_frame
+#' @importFrom sjstats resp_var model_frame var_names
 #' @importFrom rlang parse_expr
 #' @importFrom purrr map flatten_chr map_lgl
 #' @importFrom sjmisc is_empty
-get_se_from_vcov <- function(model, fitfram, typical, terms, fun = NULL, type = "fe") {
+#' @importFrom lme4 VarCorr
+safe_se_from_vcov <- function(model,
+                              fitfram,
+                              typical,
+                              terms,
+                              fun,
+                              type,
+                              vcov.fun,
+                              vcov.type,
+                              vcov.args) {
+
   mf <- sjstats::model_frame(model, fe.only = FALSE)
 
   # copy data frame with predictions
@@ -1087,7 +1529,6 @@ get_se_from_vcov <- function(model, fitfram, typical, terms, fun = NULL, type =
     terms,
     typ.fun = typical,
     fac.typical = FALSE,
-    type = type,
     pretty.message = FALSE
   )
 
@@ -1100,20 +1541,23 @@ get_se_from_vcov <- function(model, fitfram, typical, terms, fun = NULL, type =
 
   new.resp <- sjstats::var_names(sjstats::resp_var(model))
 
-  if (!sjmisc::is_empty(tidyselect::starts_with("cbind(", vars = new.resp))) {
+  if (!sjmisc::is_empty(string_starts_with(pattern = "cbind(", x = new.resp))) {
     av <- all.vars(stats::formula(model))
     get.cb <- purrr::map_lgl(av, ~ grepl(.x, new.resp, fixed = T))
     new.resp <- av[get.cb]
-    newdata <- tibble::add_column(
+    newdata <- add_cols(
       newdata,
-      response.val1 = 0,
-      response.val2 = 0
+      data.frame(
+        response.val1 = 0,
+        response.val2 = 0
+      ),
+      .after = -1
     )
-    colnames(newdata)[(ncol(newdata) - 1):ncol(newdata)] <- new.resp
+    colnames(newdata)[1:2] <- new.resp
   } else {
-    newdata <- tibble::add_column(newdata, response.val = 0)
+    newdata <- add_cols(newdata, data.frame(response.val = 0), .after = -1)
     # proper column names, needed for getting model matrix
-    colnames(newdata)[ncol(newdata)] <- new.resp
+    colnames(newdata)[1] <- new.resp
   }
 
 
@@ -1138,19 +1582,29 @@ get_se_from_vcov <- function(model, fitfram, typical, terms, fun = NULL, type =
   newdata <- dplyr::arrange(newdata, !! trms)
   fitfram <- dplyr::arrange(fitfram, !! trms)
 
-  # get variance-covariance-matrix, depending on model type
-  if (is.null(fun))
-    vcm <- as.matrix(stats::vcov(model))
-  else if (fun %in% c("hurdle", "zeroinfl"))
-    vcm <- as.matrix(stats::vcov(model, model = "count"))
-  else if (fun == "betareg")
-    vcm <- as.matrix(stats::vcov(model, model = "mean"))
-  else if (fun == "truncreg") {
-    vcm <- as.matrix(stats::vcov(model))
-    # remove sigma from matrix
-    vcm <- vcm[1:(nrow(vcm) - 1), 1:(ncol(vcm) - 1)]
-  } else
-    vcm <- as.matrix(stats::vcov(model))
+
+  # check if robust vcov-matrix is requested
+  if (!is.null(vcov.fun)) {
+    if (!requireNamespace("sandwich", quietly = TRUE)) {
+      stop("Package `sandwich` needed for this function. Please install and try again.")
+    }
+    vcov.fun <- get(vcov.fun, asNamespace("sandwich"))
+    vcm <- as.matrix(do.call(vcov.fun, c(list(x = model, type = vcov.type), vcov.args)))
+  } else {
+    # get variance-covariance-matrix, depending on model type
+    if (is.null(fun))
+      vcm <- as.matrix(stats::vcov(model))
+    else if (fun %in% c("hurdle", "zeroinfl")) {
+      vcm <- as.matrix(stats::vcov(model, model = "count"))
+    } else if (fun == "betareg")
+      vcm <- as.matrix(stats::vcov(model, model = "mean"))
+    else if (fun == "truncreg") {
+      vcm <- as.matrix(stats::vcov(model))
+      # remove sigma from matrix
+      vcm <- vcm[1:(nrow(vcm) - 1), 1:(ncol(vcm) - 1)]
+    } else
+      vcm <- as.matrix(stats::vcov(model))
+  }
 
 
   # here we need to fix some term names, so variable names
@@ -1169,6 +1623,13 @@ get_se_from_vcov <- function(model, fitfram, typical, terms, fun = NULL, type =
   mm <- stats::model.matrix(stats::terms(model), newdata)
   mmdf <- as.data.frame(mm)
   mm.rows <- as.numeric(rownames(unique(mmdf[intersect(colnames(mmdf), terms)])))
+
+  # for poly-terms, we have no match, so fix this here
+  if (sjmisc::is_empty(mm.rows) || !all(terms %in% colnames(mmdf))) {
+    inters <- which(sjstats::var_names(colnames(mmdf)) %in% terms)
+    mm.rows <- as.numeric(rownames(unique(mmdf[inters])))
+  }
+
   mm <- mm[mm.rows, ]
 
   if (!is.null(fun) && fun %in% c("polr", "multinom")) {
@@ -1183,13 +1644,12 @@ get_se_from_vcov <- function(model, fitfram, typical, terms, fun = NULL, type =
   # condition on random effect variances
   if (type == "re") {
     sig <- 0
-    if (inherits(model, c("merMod", "lmerMod", "glmerMod")) &&
-        requireNamespace("lme4", quietly = TRUE)) {
+    if (inherits(model, c("merMod", "lmerMod", "glmerMod"))) {
       sig <- sum(attr(lme4::VarCorr(model), "sc"))
     } else if (inherits(model, c("lme", "nlme"))) {
       sig <- model$sigma
     }
-    pvar <- pvar + sig^2
+    pvar <- pvar + sig^2 + getVarRand(model)
   }
 
   se.fit <- sqrt(pvar)
